package messageworker

import (
	"encoding/json"
	"os"
	"time"

	"github.com/go-resty/resty/v2"
	"jonasled.dev/jonasled/ems-esp-logger/database"
	"jonasled.dev/jonasled/ems-esp-logger/database/tables"
	"jonasled.dev/jonasled/ems-esp-logger/helper"
	"jonasled.dev/jonasled/ems-esp-logger/log"
	"jonasled.dev/jonasled/ems-esp-logger/queue"
	"jonasled.dev/jonasled/ems-esp-logger/types"
)

func RunClient() {
	if os.Getenv("CLIENT_USE_SERVER") != "true" {

		log.Log.Info("Intialized local client with direct datbase connection")
		runDbClient()
	} else {
		log.Log.Info("Using remote server as data storage")
		runHttpClient()
	}
}

func RunServer() {
	runDbClient()
}

func runHttpClient() {
	client := resty.New()
	client.SetBaseURL(os.Getenv("API_ENDPOINT"))
	client.SetHeader("Authentication", os.Getenv("INSTANCE_APIKEY"))
	client.SetHeader("X-Instance", os.Getenv("INSTANCE_NAME"))

	for {
		taskData, taskId, err := queue.MainQueue.Dequeue()
		if err != nil {
			time.Sleep(time.Second)
			continue
		}
		log.Log.Debug("Received new task: ", taskData)
		var task types.Task
		err = json.Unmarshal([]byte(taskData), &task)
		if err != nil {
			log.Log.Error("Failed reading task data, discarding task:  ", err.Error())
			continue
		}
		resp, err := client.R().SetBody(task).Post("/api/data")
		if err != nil {
			log.Log.Error("Failed uploading task to server: ", err.Error())
			queue.MainQueue.Enqueue(taskData, 60)
			queue.MainQueue.MarkDone(taskId)
			continue
		}

		if resp.StatusCode() != 200 {
			log.Log.Error("Received invalid status code from server ", resp.StatusCode(), " response: ", string(resp.Body()))
			queue.MainQueue.Enqueue(taskData, 60)
			queue.MainQueue.MarkDone(taskId)
			continue
		}

		err = queue.MainQueue.MarkDone(taskId)
		if err != nil {
			log.Log.Error("Failed marking element in queue as done: ", err.Error())
		}
		log.Log.Info("Successfully uploaded dataset to server")
	}
}

func runDbClient() {
	for {
		taskData, taskId, err := queue.MainQueue.Dequeue()
		if err != nil {
			time.Sleep(time.Second)
			continue
		}
		log.Log.Debug("Received new task: ", taskData)
		var task types.Task
		err = json.Unmarshal([]byte(taskData), &task)
		if err != nil {
			log.Log.Error("Failed reading task data, discarding task:  ", err.Error())
			queue.MainQueue.MarkDone(taskId)
			continue
		}

		var instance tables.Instance
		err = database.Db.Where("name = ?", task.Instance).First(&instance).Error
		if err != nil {
			log.Log.Error("Failed retreiving instance from database, pushing task back to queue: ", err.Error())
			queue.MainQueue.Enqueue(taskData, 60)
			queue.MainQueue.MarkDone(taskId)
			continue
		}

		var jsonData map[string]interface{}
		err = json.Unmarshal([]byte(task.Data), &jsonData)
		if err != nil {
			log.Log.Error("Failed decoding boiler JSON: ", err.Error())
			queue.MainQueue.Enqueue(taskData, 60)
			queue.MainQueue.MarkDone(taskId)
			continue
		}
		valuesToInsert := []tables.Value{}
		for key, value := range jsonData {
			valueType := database.GetOrCreateValueType(key)
			dbValue := tables.Value{
				Date:      task.Date,
				ValueType: valueType,
				Value:     helper.AnyToString(value),
				Instance:  instance,
			}
			valuesToInsert = append(valuesToInsert, dbValue)
		}
		database.Db.Create(&valuesToInsert)
		err = queue.MainQueue.MarkDone(taskId)
		if err != nil {
			log.Log.Error("Failed marking element in queue as done: ", err.Error())
		}
		log.Log.Info("Stored boiler data in database")
	}
}
